import tensorflow as tf
import os
from utils.wrappers import *
from abc import ABC, abstractmethod



class LatentAlgo(ABC):
    def __init__(self, config):
        # tensorboard writer
        tf_dir = config['logdir'] / 'tensorboard'
        self._writer = tf.summary.create_file_writer(str(tf_dir), max_queue=1000, flush_millis=20000)
        self._writer.set_as_default()

    def save_actor_critic(self, filename):
        if not os.path.exists(filename):
            os.mkdir(filename)
        self._actor.save(filename)
        self._critic.save(filename)

    def load_actor(self, filename):
        self._actor.load(filename)
        print('A trained actor has been loaded.')

    def load_actor_critic(self, filename):
        self._actor.load(filename)
        self._critic.load(filename)

    def _write_summaries(self, metrics, step=None):
        step = int(step)
        metrics = [(k, float(v)) for k, v in metrics.items()]
        with self._writer.as_default():
            tf.summary.experimental.set_step(step)
            [tf.summary.scalar(k, m, step=step) for k, m in metrics]
        print(f'[{step}]', ' / '.join(f'{k} {v:.1f}' for k, v in metrics))
        self._writer.flush()

    @abstractmethod
    def actor_critic_train_step(self, data):
        pass

    @abstractmethod
    def select_action(self, feat):
        pass






